#!/usr/bin/perl

use strict;
use warnings;
use Data::Dumper;
use WWW::Mechanize;
use Encode;

my $nif = shift || die "Usage: $0 <nif>\n";

my $url = 'http://ec.europa.eu/taxation_customs/vies/vieshome.do?selectedLanguage=EN';
my $mech;

eval {
  $mech = WWW::Mechanize->new( timeout => 15 );
  $mech->get( $url );

  $mech->submit_form(
    form_name => 'vowRequestForm',
    fields    => {
      memberStateCode  => 'PT',
      number => $nif,
    },
    button    => 'check',
  );
};

exit if $@;

my $cnt = $mech->content();

exit unless $cnt;

$cnt =~ s/<tr>.*?<td/<tr><td>/gms;

my( $address ) = $cnt =~ /<tr><td> class=".*?">Address<\/td>.*?<td>(.*?)<\/td>/ms;
my( $name ) = $cnt =~ /<tr><td> class=".*?">Name<\/td>.*?<td>(.*?)<\/td>/ms;

if ($name) {
  $name =~ s/<br \/>/#/g;
  $name =~ s/[\r\n]+$//;
  $name = trim($name);
}
else {
  $name = "";
}

if ($address) {
  $address =~ s/<br \/>/#/g;
  $address =~ s/[\r\n]+$//;
  $address = trim($address);
}
else {
  $address = "";
}

if ($name) {
  print encode("utf8", "$name#$address");
}

sub trim {
  my $w = shift;
  return $w unless defined $w && $w;
  $w =~ s/^\s+//;
  $w =~ s/\s+$//;
  return $w;
}

__END__
use lib "/opt/pos/lib";
use Data::Dumper;
use LWP::Simple qw( get );
use JSON::XS qw( decode_json );
use App::POS::Utils qw( my_conv );
use Encode;


my $key = shift;
my $nif = shift;
die "Usage: $0 <ket> <nif>\n" unless defined $nif && defined $key;

my $url = 'http://www.nif.pt/?json=1&key='.$key.'&q='.$nif.'"';

my $cnt = get($url);
my $ref = decode_json($cnt);

$ref->{result} ||= "";
die "ERROR\n" unless $ref->{result} eq 'success';

my $name = encode("utf8", $ref->{records}->{$nif}->{title});
my $address = encode("utf8", $ref->{records}->{$nif}->{address});
my $pcode = encode("utf8", $ref->{records}->{$nif}->{pc4})."-".encode("utf8", $ref->{records}->{$nif}->{pc3});
my $city = encode("utf8", $ref->{records}->{$nif}->{city});

print $name."#".$address."#".$pcode."#".$city;

